<?php

namespace Zyimm\MmdsOss;


use Carbon\Carbon;
use finfo;
use GuzzleHttp\Exception\GuzzleException;
use Hyperf\Contract\ConfigInterface;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Guzzle\ClientFactory;
use Hyperf\HttpServer\Contract\RequestInterface;
use Hyperf\Logger\LoggerFactory;
use Hyperf\Utils\ApplicationContext;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use Psr\Log\LoggerInterface;
use Zyimm\MmdsOss\Exception\OssException;

class Oss
{
    public array $oss = [];

    public ?string $authorization = null;
    /**
     * @Inject()
     * @var ClientFactory
     */
    public ClientFactory $clientFactory;

    /**
     * @Inject
     * @var RequestInterface
     */
    public RequestInterface $request;


    /**
     * @Inject
     *
     * @var ConfigInterface
     */
    public ConfigInterface $config;


    /**
     * upload
     *
     * @return array
     * @throws ContainerExceptionInterface
     * @throws GuzzleException
     * @throws NotFoundExceptionInterface
     */
    public function upload(): array
    {
        $file         = $this->request->file('file');
        $file_content = file_get_contents($file->getPathname());
        $fInfo        = new finfo(FILEINFO_MIME_TYPE);
        $mimeType     = $fInfo->buffer($file_content);
        $file_info    = [
            'name'     => 'file',
            'contents' => fopen($file->getPathname(), 'r'),
            'filename' => Carbon::now()->getTimestampMs().uniqid().'.'.$file->getExtension(),

        ];
        $result       = $this->getSecretKey()->bucket()->ossUpload($file_info);
        return [
            'url'       => $result['file_url'] ?? '',
            'size'      => $result['file_size'] ?? 0,
            'mime_type' => $mimeType
        ];
    }

    /**
     * getUploadAttr
     *
     * @return array
     * @throws ContainerExceptionInterface
     * @throws GuzzleException
     * @throws NotFoundExceptionInterface
     */
    public function getUploadAttr(): array
    {
        $this->getSecretKey()->bucket();
        return [
            'timestamp'                 => time(),
            'filestorage-authorization' => $this->authorization,
            'bucket'                    => $this->getBucketName()
        ];
    }

    /**
     * ossUpload
     *
     * @param $file_info
     * @return array
     * @throws GuzzleException
     */
    private function ossUpload($file_info): array
    {
        $client = $this->clientFactory->create([
            'base_uri' => $this->getBaseUri(),
            'headers'  => array_merge($this->request->getHeaders(), [
                'filestorage-authorization' => $this->authorization,
                'timestamp'                 => time()
            ])
        ]);
        $path   = 'backend/normalUpload';

        $response = $client->request($this->request->getMethod(), $path, [
            'multipart' => [
                $file_info,
                [
                    'name'     => 'object_key',
                    'contents' => $file_info['filename']
                ],
                [
                    'name'     => 'bucket',
                    'contents' => $this->getBucketName()
                ]
            ]
        ]);

        $data = json_decode($response->getBody()->getContents(), true);
        if (isset($data['error_code']) && (int) $data['error_code'] == 200) {
            return $data['result'] ?? [];
        } else {
            throw new OssException($data['error_message'] ?? 'oss upload error',
                $data['error_code'] ?? __LINE__);
        }
    }

    private function getBaseUri()
    {
        return $this->config->get('mmds_oss.xy_oss_url');
    }

    private function getBucketName()
    {
        return $this->config->get('mmds_oss.xy_oss_bucket_name');
    }

    /**
     * bucket
     *
     * @return Oss
     * @throws ContainerExceptionInterface
     * @throws GuzzleException
     * @throws NotFoundExceptionInterface
     */
    public function bucket(): Oss
    {
        $this->getSecretKey();
        $time                = time();
        $this->authorization = $this->oss['access_key_id'].':'.base64_encode(
                hash_hmac('sha1',
                    $this->oss['secret_key'],
                    (string) $time, true));
        $bucket              = $this->getBucketName();
        $client              = $this->clientFactory->create([
            'base_uri' => $this->getBaseUri(),
            'headers'  => [
                'filestorage-authorization' => $this->authorization,
                'timestamp'                 => $time,
                'Content-Type'              => 'application/json'
            ]
        ]);
        $path                = '/backend/bucket';
        $response            = $client->request('POST', $path, [
            'json' => [
                'bucket_name' => $bucket
            ],
        ]);
        $data                = json_decode($response->getBody()->getContents(), true);
        if (isset($data['error_code']) && (int) $data['error_code'] == 200) {
            $this->logger()->info('oss', $data);
        } else {
            throw new OssException($data['error_message'] ?? 'oss upload error',
                $data['error_code'] ?? __LINE__);
        }
        return $this;
    }

    /**
     * getSecretKey
     *
     * @throws GuzzleException
     */
    public function getSecretKey(): Oss
    {

        if (!$this->oss) {
            $client = $this->clientFactory->create([
                'base_uri' => $this->getBaseUri(),
                'headers'  => $this->request->getHeaders()
            ]);
            $path   = '/backend/secretKey?'.http_build_query([
                    'service_name' => $this->config->get('mmds_oss.xy_oss_service_name'),
                    'group_name'   => $this->config->get('mmds_oss.xy_oss_group_name'),
                ]);

            $response = $client->request('GET', $path);
            $data     = json_decode($response->getBody()->getContents(), true);
            if (isset($data['error_code']) && (int) $data['error_code'] == 200) {
                $this->oss = [
                    'access_key_id' => $data['result']['accessKeyId'] ?? null,
                    'secret_key'    => $data['result']['secretKey'] ?? null
                ];

            } else {
                throw new OssException($data['error_message'] ?? 'oss error',
                    $data['error_code'] ?? __LINE__);
            }
        }
        return $this;
    }


    /**
     * logger
     *
     * @return LoggerInterface
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function logger(): LoggerInterface
    {
        return ApplicationContext::getContainer()->get(LoggerFactory::class)->get('oss');
    }
}